@@ -8,6 +8,7 @@ from typing import (
     Callable,
     Dict,
     List,
+    Literal,
     Mapping,
     MutableSequence,
     Optional,
@@ -18,7 +19,6 @@ from typing import (
 )
 
 import numpy as np
-from typing_extensions import Literal
 
 import aesara
 from aesara.compile.ops import ViewOp
@@ -91,10 +91,8 @@ def grad_not_implemented(op, x_pos, x, comment=""):
 
     return (
         NullType(
-            (
-                "This variable is Null because the grad method for "
-                f"input {x_pos} ({x}) of the {op} op is not implemented. {comment}"
-            )
+            "This variable is Null because the grad method for "
+            f"input {x_pos} ({x}) of the {op} op is not implemented. {comment}"
         )
     )()
 
@@ -114,10 +112,8 @@ def grad_undefined(op, x_pos, x, comment=""):
 
     return (
         NullType(
-            (
-                "This variable is Null because the grad method for "
-                f"input {x_pos} ({x}) of the {op} op is not implemented. {comment}"
-            )
+            "This variable is Null because the grad method for "
+            f"input {x_pos} ({x}) of the {op} op is not implemented. {comment}"
         )
     )()
 
@@ -229,7 +225,6 @@ def Rop(
     # Check that each element of wrt corresponds to an element
     # of eval_points with the same dimensionality.
     for i, (wrt_elem, eval_point) in enumerate(zip(_wrt, _eval_points)):
-
         try:
             if wrt_elem.type.ndim != eval_point.type.ndim:
                 raise ValueError(
@@ -266,7 +261,6 @@ def Rop(
                     # arguments, like for example random states
                     local_eval_points.append(None)
             elif inp.owner in seen_nodes:
-
                 local_eval_points.append(
                     seen_nodes[inp.owner][inp.owner.outputs.index(inp)]
                 )
@@ -687,9 +681,9 @@ def subgraph_grad(wrt, end, start=None, cost=None, details=False):
         w2 = aesara.shared(np.random.standard_normal((4,2)))
         a1 = aesara.tensor.tanh(aesara.tensor.dot(x,w1))
         a2 = aesara.tensor.tanh(aesara.tensor.dot(a1,w2))
-        cost2 = aesara.tensor.sqr(a2 - t).sum()
-        cost2 += aesara.tensor.sqr(w2.sum())
-        cost1 = aesara.tensor.sqr(w1.sum())
+        cost2 = aesara.tensor.square(a2 - t).sum()
+        cost2 += aesara.tensor.square(w2.sum())
+        cost1 = aesara.tensor.square(w1.sum())
 
         params = [[w2],[w1]]
         costs = [cost2,cost1]
@@ -941,7 +935,6 @@ def _populate_var_to_app_to_idx(outputs, wrt, consider_constant):
             var_idx = app.outputs.index(var)
 
             for i, ipt in enumerate(app.inputs):
-
                 # don't process ipt if it is not a true
                 # parent of var
                 if not connection_pattern[i][var_idx]:
@@ -1052,7 +1045,6 @@ def _populate_grad_dict(var_to_app_to_idx, grad_dict, wrt, cost_name=None):
         """Populates term_dict[node] and returns it"""
 
         if node not in term_dict:
-
             inputs = node.inputs
 
             output_grads = [access_grad_cache(var) for var in node.outputs]
@@ -1267,7 +1259,6 @@ def _populate_grad_dict(var_to_app_to_idx, grad_dict, wrt, cost_name=None):
             ]
 
             for i, term in enumerate(input_grads):
-
                 # Disallow Nones
                 if term is None:
                     # We don't know what None means. in the past it has been
@@ -1275,14 +1266,12 @@ def _populate_grad_dict(var_to_app_to_idx, grad_dict, wrt, cost_name=None):
                     # We therefore don't allow it because its usage has become
                     # so muddied.
                     raise TypeError(
-                        (
-                            f"{node.op}.grad returned None for a gradient term, "
-                            "this is prohibited. Instead of None,"
-                            "return zeros_like(input), disconnected_type(),"
-                            " or a NullType variable such as those made with "
-                            "the grad_undefined or grad_unimplemented helper "
-                            "functions."
-                        )
+                        f"{node.op}.grad returned None for a gradient term, "
+                        "this is prohibited. Instead of None,"
+                        "return zeros_like(input), disconnected_type(),"
+                        " or a NullType variable such as those made with "
+                        "the grad_undefined or grad_unimplemented helper "
+                        "functions."
                     )
 
                 # Check that the gradient term for this input
@@ -1383,7 +1372,6 @@ def _populate_grad_dict(var_to_app_to_idx, grad_dict, wrt, cost_name=None):
                 node_to_idx = var_to_app_to_idx[var]
                 for node in node_to_idx:
                     for idx in node_to_idx[node]:
-
                         term = access_term_cache(node)[idx]
 
                         if not isinstance(term, Variable):
@@ -1402,10 +1390,8 @@ def _populate_grad_dict(var_to_app_to_idx, grad_dict, wrt, cost_name=None):
 
                         if hasattr(var, "ndim") and term.ndim != var.ndim:
                             raise ValueError(
-                                (
-                                    f"{node.op}.grad returned a term with"
-                                    f" {int(term.ndim)} dimensions, but {int(var.ndim)} are required."
-                                )
+                                f"{node.op}.grad returned a term with"
+                                f" {int(term.ndim)} dimensions, but {int(var.ndim)} are required."
                             )
 
                         terms.append(term)
@@ -1767,10 +1753,8 @@ def verify_grad(
     for i, p in enumerate(pt):
         if p.dtype not in ("float16", "float32", "float64"):
             raise TypeError(
-                (
-                    "verify_grad can work only with floating point "
-                    f'inputs, but input {i} has dtype "{p.dtype}".'
-                )
+                "verify_grad can work only with floating point "
+                f'inputs, but input {i} has dtype "{p.dtype}".'
             )
 
     _type_tol = dict(  # relative error tolerances for different types
@@ -1868,7 +1852,6 @@ def verify_grad(
         )
 
         if max_abs_err > abs_tol and max_rel_err > rel_tol:
-
             raise GradientError(
                 max_arg,
                 max_err_pos,
@@ -2052,7 +2035,6 @@ def hessian(cost, wrt, consider_constant=None, disconnected_inputs="raise"):
 
     hessians = []
     for input in wrt:
-
         if not isinstance(input, Variable):
             raise TypeError("hessian expects a (list of) Variable as `wrt`")
 
